from Tkinter import *

#####
# The GridWindow class can be used to generate Sudoku grids of
# various sizes.
class GridWindow:
    
    # Creates a sudoko grid of the specified size
    def __init__(self, master, row, col):
        
        self.row = row
        self.col = col
        
        self.master = master
        self.window = Toplevel(master)

        self.window.bind("<Button-1>", self.onClick)
        self.window.bind("<Key>", self.onKey)
        self.window.protocol("WM_DELETE_WINDOW", self.onClose)

        self.canvas = Canvas(self.window, width=row*40+1, height=col*40+1)
        self.canvas.pack()
                
        self.rectangle = {}
        self.text = {}
        
        # The square that currently has focus. A square is stored as a string,
        # in the format NxN where N is an integer from 0 to 9
        self.focus = ""
        
        # These are the keyboard keys that can be used to clear a
        # square. Right now, there is space, backspace and enter
        self.clearkey = [13, 8, 32]
    
        for i in range(0,self.row):
            for j in range(0,self.col):
                self.rectangle[str(i) + "x" + str(j)] = self.canvas.create_rectangle(2+40*i, 2+40*j, 42+40*i, 42+40*j, fill="White")                                
                self.text[str(i) + "x" + str(j)] = self.canvas.create_text(22+40*i, 22+40*j, text="")
    
    # Sets a number in the specified square  
    def setNumber(self, location, number):
        itemId = self.text[location]
        self.canvas.itemconfigure(itemId, text=number)
    
    # Sets the color of the specified square
    def setColor(self, location, color):        
        itemId = self.rectangle[location]
        self.canvas.itemconfigure(itemId, fill=color)
    
    # Sets the color of a given square
    def onClick(self, event):        
        x = (event.x - 2) / 40
        y = (event.y - 2) / 40
                
        if x >= 0 and x < self.row and x >= 0 and y < self.col:
            if (self.focus != ""):
                self.setColor(self.focus, "White")                
            self.focus = str(x) + "x" + str(y)
            self.setColor(self.focus, "LightGray")
    
    # This function is called when a keyboard key is pressed when
    # this sudoku window has focus.
    def onKey(self, event):
        if self.focus != "":
            if "123456789".find(event.char) != -1:
                newValue = event.char            
            elif event.keycode in self.clearkey:
                newValue = ""                
            else:
                return
            
            # Here, I'm directly editing the grid, as an example. However, you
            # guys shouldn't do this. You probably need to give the change
            # to some other object first.
            # Put your code here
            self.setNumber(self.focus, newValue)
    
    # This function is called then a particular window is closed.
    def onClose(self):
        self.window.destroy()
            
class MainWindow(GridWindow):
    
    def __init__(self, master):
        
        GridWindow.__init__(self, master,9,9)
        
        for i in range(0,2):
            self.canvas.create_line(122+120*i, 2, 122+120*i, 361, width=3)
            self.canvas.create_line(2, 122+120*i, 361, 122+120*i, width=3)

#####
# Creates a 1 by 9 Sudoku grid, suitable for displaying a row.
class RowWindow(GridWindow):
    
    def __init__(self, master, id):
        
        GridWindow.__init__(self, master, 1, 9)

#####
# Creates a 9 by 1 Sudoku grid, suitable for displaying a column.        
class ColWindow(GridWindow):
    
    def __init__(self, master, id):
        
        GridWindow.__init__(self, master, 9, 1)

#####
# Creates a 3 by 3 Sudoku grid, suitable for displaying a block.
class BlockWindow(GridWindow):
    
    def __init__(self, master, id):
        
        GridWindow.__init__(self, master, 3, 3)
                